/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* $Id: imaging_fdata.h,v 1.4 2002/09/04 08:27:35 jdejong Exp $ */
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: imaging_fdata.h,v 1.4 2002/09/04 08:27:35 jdejong Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jdejong 2002-05-08  imported
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef _IMAGING_FDATA_H
#define _IMAGING_FDATA_H
/*----------------------------------------------------------------------- */
/* Imaging data FITS binary tables routines                               */
/* Raw data table (float version ) for optical/IR interferometer          */
/*----------------------------------------------------------------------- */
#include "VLTItable.h"
#include "imaging_detector.h"

/* set type of data */
typedef float fDataType;
/* corresponding fitsio code */
#define FDATACODE TFLOAT

typedef struct      /* table definition for imaging data table */
{
  table_def *def;      /* basic table information */
  imaging_detector_table_def *detector; /* descriptor for this table */
  long   row;          /* last row (0-rel) read or written, -1=> none. */
  char   *date_obs;    /* observing date */
  int    nregion;      /* Number of regions  */
  int    maxstep;      /* MAXSTEP keyword */
  int    frame_col;    /* frame column number (0-rel)*/
  int    time_col;     /* time column number (0-rel)*/
  int    exp_col;      /* exposure column number */
  int    opt_tran_col; /* opt_tran column numbers */
  int    opd_col;      /* OPD column number */
  int    local_opd_col;/* local OPD column number */
  int    offset_col;   /* offset column number */
  int    rotation_col; /* rotation column number */
  int    reference_col;/* source column number */
  int    step_col;     /* step column number */
  int    ins_train_col; /* instrument train column number */
  int    *targtype_col;/* targtype column numbers */
  int    *target_col;  /* source column numbers */
  int    *data_col;    /* data column numbers */
} imaging_fdata_table_def;

typedef struct      /* structure used to imaging data row */
{
  double time;       /* central time of exposure */
  double opd[2];        /* Primary delay */
  double local_opd[2];  /* Secondary delay */
  float  exp;        /* exposure length */
  float  offset[2];  /* position offset from source position */
  float  rotation;   /* rotation angle on sky */
  long   frame;      /* frame number */
  int    reference;  /* source id */
  int    step;       /* step number in observing cycle */
  int    nregion;    /* number of regions of data */
  int    *opt_train; /* optical train numbers contributing to the measurment */
  char   *targtype;  /* data type 'T'=target, 'R'=reference, 'S'=sky, 'B'=background 
                                  'U'=unknown */
  int    *target;    /* source number for each region */
                     /* one value per region */
  int    *ins_train; /* description of used optical elements */
  fDataType  **data;  /* array of data arrays, one per region */
  table_row *raw;    /* generic table row */
} imaging_fdata_row;

/* public functions */
/* there are more complete descriptions of the functions in the source file */

/* imaging_fdata_row functions */

imaging_fdata_row* make_imaging_fdata_row(imaging_fdata_table_def* table);
/*----------------------------------------------------------------------- */
/*   Make an imaging_fdata_row structure                                  */
/*   Inputs:                                                              */
/*      table     pointer to imaging_fdata table structure                */
/*   Returns:                                                             */
/*      An imaging_fdata_row structure pointer ; NULL indicates failure   */
/*      Data pointers set to generic structure values                     */
/*----------------------------------------------------------------------- */


void kill_imaging_fdata_row (imaging_fdata_row *row);
/*----------------------------------------------------------------------- */
/*   Delete an imaging_fdata_row structure                                */
/*   Inputs:                                                              */
/*      row    pointer to the imaging_fdata_row structure.                */
/*----------------------------------------------------------------------- */

/* imaging_fdata_table functions */
imaging_fdata_table_def* 
make_imaging_fdata_table_def(fitsfile *fptr, 
			    imaging_detector_table_def *detector,
			    int maxstep, int *ierr);
/*----------------------------------------------------------------------- */
/*   Make a new imaging_fdata_table in an open FITS file                  */
/*   Writes imaging detector table to FITS file;                          */
/*   then opens imaging data table                                        */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      detector  imaging_detector_table describing table                 */
/*      maxstep   Maximum number of steps in obs. sequence.               */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      An imaging_fdata_table_def pointer; NULL indicates failure        */
/*----------------------------------------------------------------------- */

imaging_fdata_table_def* 
open_imaging_fdata_table(fitsfile *fptr, int *ierr);
/*----------------------------------------------------------------------- */
/*   Open an existing imaging_fdata_table in the active FITS file         */
/*   Reads imaging_detector_table and saves in imaging_fdata_table_def    */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      An imaging_fdata_table_def pointer; NULL indicates failure        */
/*----------------------------------------------------------------------- */

int read_imaging_fdata_table(fitsfile *fptr,  imaging_fdata_table_def* table, 
			    imaging_fdata_row *row, long row_number,
			    int *ierr);
/*----------------------------------------------------------------------- */
/*   Read the next row from an imaging_fdata_table                        */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      table     table definitions structure from open or create         */
/*      row_number which row (1-rel) in the table                         */   
/*                 -1 => next row                                         */  
/*   Input/output:                                                        */
/*      row       pointer to an extant row structure                      */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */

int write_imaging_fdata_table(fitsfile *fptr, imaging_fdata_table_def* table, 
			     imaging_fdata_row *row,  long row_number,
			     int *ierr);
/*----------------------------------------------------------------------- */
/*   Write  the next row to an imaging_fdata_table                        */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      table     table definitions structure from open or create         */
/*      row       pointer to an extant row structure                      */
/*      row_number which row (1-rel) in the table                         */     
/*                 -1 => next row                                         */  
/*   Input/output:                                                        */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */

int close_imaging_fdata_table(fitsfile *fptr, imaging_fdata_table_def* table, 
			     int *ierr);
/*----------------------------------------------------------------------- */
/*   Close an open imaging_fdata_table, associated structures are destroyed*/
/*   Note: this closes the full FITS file                                 */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */

void kill_imaging_fdata_table_def (imaging_fdata_table_def* table);
/*----------------------------------------------------------------------- */
/*   Delete an imaging_fdata_table structure                              */
/*   Inputs:                                                              */
/*      table     table definitions structure from open or create         */
/*----------------------------------------------------------------------- */

#endif /* end IMAGING_FDATA_H */
/* */

